import 'dart:convert';
import 'package:modular_system_core/mscore.dart';
import 'package:http/http.dart' as http;

class ApiController<T extends JsonSerializer<T>> extends WebController {
  ApiController({
    required String baseUri,
    required this.controllerName,
    required this.objectCreator,
    this.token = "",
  }) : super(baseUri: baseUri);

  final String controllerName;
  final String token;
  final JsonObjectCreator<T> objectCreator;

  @override
  String getUri() {
    return "$baseUri/api/$controllerName";
  }

  Future<StringDTO> createAsync(T object) async {
    var resp = await http.post(
      Uri.parse(getUri()),
      headers: <String, String>{
        'Content-Type': 'application/json',
        'cookie': token,
      },
      body: object.toJson(),
    );
    var jsonMap = getJsonFrom(resp);
    return StringDTO.fromJson(jsonMap);
  }

  // READ
  Future<T> getByIdAsync(String id) async {
    var resp = await http.get(
      Uri.parse("$getUri()?id=$id"),
      headers: <String, String>{
        'Content-Type': 'application/json; charset=UTF-8',
        'cookie': token,
      },
    );
    var jsonMap = getJsonFrom(resp);
    return objectCreator(jsonMap);
  }

  Future<QueryList<T>> searchAsync(WebSearchRequest request) async {
    var resp = await http.post(
      Uri.parse("$getUri()/query"),
      headers: <String, String>{
        'Content-Type': 'application/json;',
        'Access-Control-Allow-Origin': '*',
        'cookie': token,
      },
      body: utf8.encode(request.toJson()),
    );
    var jsonMap = getJsonFrom(resp);
    return QueryList.fromJsonMap(jsonMap, objectCreator);
  }

  // UPDATE
  Future updateAsync(WebUpdateRequest request) async {
    var resp = await http.put(
      Uri.parse(getUri()),
      headers: <String, String>{
        'Content-Type': 'application/json; charset=UTF-8',
        'Access-Control-Allow-Origin': '*',
        'cookie': token,
      },
      body: request.toJson(),
    );
    handleResponse(resp);
  }

  Future overrideAsync(T request) async {
    var resp = await http.put(
      Uri.parse("$getUri()/override"),
      headers: <String, String>{
        'Content-Type': 'application/json; charset=UTF-8',
        'Access-Control-Allow-Origin': '*',
        'cookie': token,
      },
      body: request.toJson(),
    );
    handleResponse(resp);
  }

  // DELETE
  Future deleteAsync(String id) async {
    var resp = await http.delete(
      Uri.parse("$getUri()?id=$id"),
      headers: <String, String>{
        'Content-Type': 'application/json',
        'cookie': token,
      },
    );
    handleResponse(resp);
  }
}
