import 'dart:convert';
import 'package:modular_system_core/mscore.dart';

abstract class WebController {
  WebController({
    required this.baseUri,
  });

  final String baseUri;
  final Map<String, String> cookies = {};

  String getUri();

  void handleResponse(Response resp) {
    if (resp.statusCode == 200) {
      return;
    } else if (resp.body.isNotEmpty) {
      throw AppException.fromJson(resp.body);
    } else {
      throw AppException("${resp.statusCode} ${resp.reasonPhrase}");
    }
  }

  void setSessionCookie(String cookie) {
    cookies["app-session"] = cookie;
  }

  String getCookieHeader() {
    var param = "";
    for (var entry in cookies.entries) {
      if (param.isEmpty) {
        param = "${entry.key}=${entry.value}";
      } else {
        param = "$param; ${entry.key}=${entry.value}";
      }
    }
    return param;
  }

  Map<String, dynamic> getJsonFrom(Response resp) {
    if (resp.statusCode == 200) {
      return jsonDecode(resp.body);
    } else if (resp.body.isNotEmpty) {
      throw AppException.fromJson(resp.body);
    } else {
      throw AppException("${resp.statusCode} ${resp.reasonPhrase}");
    }
  }
}
