import 'package:flutter/material.dart';
import 'package:modular_system_core/mscore.dart';

// DART
extension ExceptionConvert on Exception {
  AppException toAppException([StackTrace? stackTrace]) {
    return AppException(toString(), ExceptionCode.internal, stackTrace);
  }
}

extension ErrorConvert on Error {
  AppException toAppException([StackTrace? stackTrace]) {
    return AppException(
      "Um erro de codificação foi detectado, a aplicação se autodestruirá em alguns instantes... ${toString()}",
      ExceptionCode.internal,
      stackTrace,
    );
  }
}

extension UriExtender on Uri {
  /// Adds a query param and returns itself.
  Uri addQueryParam(String name, String value) {
    var queryParams = <String, dynamic>{};
    queryParams.addAll(queryParameters);
    queryParams.addAll({name: value});
    return replace(queryParameters: queryParams);
  }
}

extension MapExtender<K, V> on Map<K, V> {
  void add(K key, V value) {
    addAll({key: value});
  }
}

// FLUTTER
extension InputDecorationMerger on InputDecoration {
  InputDecoration combineWith(InputDecoration decoration) {
    return copyWith(
      alignLabelWithHint: decoration.alignLabelWithHint,
      border: decoration.border,
      constraints: decoration.constraints,
      contentPadding: decoration.contentPadding,
      counter: decoration.counter,
      counterStyle: decoration.counterStyle,
      counterText: decoration.counterText,
      disabledBorder: decoration.disabledBorder,
      enabled: decoration.enabled,
      enabledBorder: decoration.enabledBorder,
      errorBorder: decoration.errorBorder,
      errorMaxLines: decoration.errorMaxLines,
      errorStyle: decoration.errorStyle,
      errorText: decoration.errorText,
      fillColor: decoration.fillColor,
      filled: decoration.filled,
      floatingLabelBehavior: decoration.floatingLabelBehavior,
      floatingLabelStyle: decoration.floatingLabelStyle,
      focusColor: decoration.focusColor,
      focusedBorder: decoration.focusedBorder,
      focusedErrorBorder: decoration.focusedErrorBorder,
      helperMaxLines: decoration.helperMaxLines,
      helperStyle: decoration.helperStyle,
      helperText: decoration.helperText,
      hintMaxLines: decoration.hintMaxLines,
      hintStyle: decoration.hintStyle,
      hintText: decoration.hintText,
      hintTextDirection: decoration.hintTextDirection,
      hoverColor: decoration.hoverColor,
      icon: decoration.icon,
      iconColor: decoration.iconColor,
      isCollapsed: decoration.isCollapsed,
      isDense: decoration.isDense,
      label: decoration.label,
      labelStyle: decoration.labelStyle,
      labelText: decoration.labelText,
      prefix: decoration.prefix,
      prefixIcon: decoration.prefixIcon,
      prefixIconColor: decoration.prefixIconColor,
      prefixIconConstraints: decoration.prefixIconConstraints,
      prefixStyle: decoration.prefixStyle,
      prefixText: decoration.prefixText,
      semanticCounterText: decoration.semanticCounterText,
      suffix: decoration.suffix,
      suffixIcon: decoration.suffixIcon,
      suffixIconColor: decoration.suffixIconColor,
      suffixIconConstraints: decoration.suffixIconConstraints,
      suffixStyle: decoration.suffixStyle,
      suffixText: decoration.suffixText,
    );
  }
}
