import 'dart:convert';

import 'package:modular_system_core/mscore.dart';

class Address implements JsonSerializer<Address> {
  Address();

  factory Address.fromEncoded(String encodedAddress) {
    var i = Address();
    var entries = encodedAddress.split(";");
    for (int index = 0; index < entries.length; index++) {
      if (index == 0) {
        i.country = entries[index].trim();
      } else if (index == 1) {
        i.state = entries[index].trim();
      } else if (index == 2) {
        i.city = entries[index].trim();
      } else if (index == 3) {
        i.neighborhood = entries[index].trim();
      } else if (index == 4) {
        i.streetName = entries[index].trim();
      } else if (index == 5) {
        i.streetNumber = entries[index].trim();
      } else if (index == 6) {
        i.zipCode = entries[index].trim();
      }
    }
    return i;
  }

  factory Address.fromJson(Map<String, dynamic> jsonMap) {
    var i = Address();
    i.isSet = jsonMap["isSet"];
    i.country = jsonMap["country"];
    i.state = jsonMap["state"];
    i.city = jsonMap["city"];
    i.neighborhood = jsonMap["neighborhood"];
    i.streetName = jsonMap["streetName"];
    i.streetNumber = jsonMap["streetNumber"];
    i.zipCode = jsonMap["zipCode"];
    return i;
  }

  bool isSet = false;
  String country = "";
  String state = "";
  String city = "";
  String neighborhood = "";
  String streetName = "";
  String streetNumber = "";
  String zipCode = "";

  String toFullAddress() =>
      "$streetName, $streetNumber - $neighborhood, $city - $state, $zipCode - $country";

  String toFullAddressLocal([bool showZipCode = false]) =>
      "$streetName, $streetNumber - $neighborhood" +
      (showZipCode ? ", $zipCode" : "");

  String toFullAddressState([bool showZipCode = false]) =>
      "$streetName, $streetNumber - $neighborhood, $city" +
      (showZipCode ? " - $zipCode" : "");

  String toFullAddressNational([bool showZipCode = false]) =>
      "$streetName, $streetNumber - $neighborhood, $city - $state" +
      (showZipCode ? ", $zipCode" : "");

  String toEncoded() {
    return "$country;$state;$city;$neighborhood;$streetName;$streetNumber;$zipCode";
  }

  @override
  String toString() => toFullAddressNational();

  @override
  Address fromJson(Map<String, dynamic> jsonMap) {
    var i = Address();
    i.isSet = jsonMap["isSet"];
    i.country = jsonMap["country"];
    i.state = jsonMap["state"];
    i.city = jsonMap["city"];
    i.neighborhood = jsonMap["neighborhood"];
    i.streetName = jsonMap["streetName"];
    i.streetNumber = jsonMap["streetNumber"];
    i.zipCode = jsonMap["zipCode"];
    return i;
  }

  @override
  Map<String, dynamic> toJsonMap() {
    var map = {
      'isSet': isSet,
      'country': country,
      'state': state,
      'city': city,
      'neighborhood': neighborhood,
      'streetName': streetName,
      'streetNumber': streetNumber,
      'zipCode': zipCode
    };
    return map;
  }

  @override
  String toJson() => jsonEncode(toJsonMap());
}
