import 'dart:convert';

import 'package:modular_system_core/mscore.dart';

enum ExceptionCode { badRequest, invalidInput, unathorized, internal, credentialsExpired, forbbiden }

class AppException implements Exception {
  AppException(this.message, [this.code = ExceptionCode.internal, this.stackTrace]);

  String message;
  ExceptionCode code = ExceptionCode.internal;
  StackTrace? stackTrace;

  factory AppException.fromJson(String json, [StackTrace? stackTrace]) {
    var map = jsonDecode(json);
    var message = map["message"] ?? (map["errors"]).toString() ?? "Erro desconhecido";
    var code = map["code"] ?? ExceptionCode.internal.index;
    return AppException(message, ExceptionCode.values[code], stackTrace);
  }

  @override
  String toString([Environment env = Environment.production]) {
    if (code == ExceptionCode.internal) {
      if (env == Environment.development) {
        return "Erro interno, debug data '$message' StackTrace: '${stackTrace?.toString() ?? "desconhecido"}' ";
      } else {
        return "Um erro inesperado aconteceu.";
      }
    } else if (code == ExceptionCode.invalidInput) {
      if (env == Environment.development) {
        return "O servidor recusou a entrada de dados. '$message' StackTrace: '${stackTrace?.toString() ?? "desconhecido"}' ";
      } else {
        return "O servidor recusou a entrada de dados, $message";
      }
    } else {
      return message;
    }
  }
}
