import 'dart:convert';
import 'package:modular_system_core/mscore.dart';

class IdentityAccessCredential
    implements JsonSerializer<IdentityAccessCredential> {
  String userKey = "";
  IdentityRole role = IdentityRole();
  AccessPermisions permisions = AccessPermisions();

  IdentityAccessCredential();

  factory IdentityAccessCredential.fromJson(Map<String, dynamic> jsonMap) {
    var x = IdentityAccessCredential();
    x.userKey = jsonMap["userKey"];
    x.role = IdentityRole.fromJson(jsonMap["role"]);
    x.permisions = AccessPermisions.fromJson(jsonMap["permissions"]);
    return x;
  }

  @override
  IdentityAccessCredential fromJson(Map<String, dynamic> jsonMap) {
    return IdentityAccessCredential.fromJson(jsonMap);
  }

  @override
  String toJson() {
    return jsonEncode(toJsonMap());
  }

  @override
  Map<String, dynamic> toJsonMap() {
    return {
      "userKey": userKey,
      "role": role.toJsonMap(),
      "permisions": permisions.toJsonMap(),
    };
  }
}

class IdentityRole implements JsonSerializer<IdentityRole> {
  String mainRole = "";
  List<String> subRoles = [];

  IdentityRole();

  factory IdentityRole.fromJson(Map<String, dynamic> jsonMap) {
    var x = IdentityRole();
    x.mainRole = jsonMap["mainRole"];
    for (var role in jsonMap["subRoles"]) {
      x.subRoles.add(role);
    }
    return x;
  }

  @override
  IdentityRole fromJson(Map<String, dynamic> jsonMap) {
    return IdentityRole.fromJson(jsonMap);
  }

  @override
  String toJson() {
    return jsonEncode(toJsonMap());
  }

  @override
  Map<String, dynamic> toJsonMap() {
    return {
      "mainRole": mainRole,
      "subRoles": subRoles,
    };
  }
}

class AccessPermision implements JsonSerializer<AccessPermision> {
  bool create = false;
  bool read = false;
  bool update = false;
  bool delete = false;

  AccessPermision();

  factory AccessPermision.fromJson(Map<String, dynamic> jsonMap) {
    var x = AccessPermision();
    x.create = jsonMap["create"];
    x.read = jsonMap["read"];
    x.update = jsonMap["update"];
    x.delete = jsonMap["delete"];
    return x;
  }

  @override
  AccessPermision fromJson(Map<String, dynamic> jsonMap) {
    return AccessPermision.fromJson(jsonMap);
  }

  @override
  String toJson() {
    return jsonEncode(toJsonMap());
  }

  @override
  Map<String, dynamic> toJsonMap() {
    return {
      "create": create,
      "read": read,
      "update": update,
      "delete": delete,
    };
  }
}

class AccessPermisions implements JsonSerializer<AccessPermisions> {
  AccessPermision owned = AccessPermision();
  AccessPermision thirdParty = AccessPermision();

  AccessPermisions();

  factory AccessPermisions.fromJson(Map<String, dynamic> jsonMap) {
    var x = AccessPermisions();
    x.owned = AccessPermision.fromJson(jsonMap["owned"]);
    x.thirdParty = AccessPermision.fromJson(jsonMap["thirdParty"]);
    return x;
  }

  @override
  AccessPermisions fromJson(Map<String, dynamic> jsonMap) {
    return AccessPermisions.fromJson(jsonMap);
  }

  @override
  String toJson() {
    return jsonEncode(toJsonMap());
  }

  @override
  Map<String, dynamic> toJsonMap() {
    return {
      "owned": owned.toJsonMap(),
      "thirdParty": thirdParty.toJsonMap(),
    };
  }
}
