import 'dart:convert';
import 'package:modular_system_core/mscore.dart';
import 'package:modular_system_core/src/model/phonte_number.dart';

enum Gender {
  unknown,
  male,
  female,
}

class PersonData implements JsonSerializer<PersonData> {
  PersonData();

  factory PersonData.fromJson(Map<String, dynamic> jsonMap) {
    var i = PersonData();
    i.age = jsonMap["age"];
    i.gender = Gender.values[jsonMap["gender"]];
    i.name = PersonName.fromJson(jsonMap["name"]);
    i.dateOfBirth = DateOfBirth.fromJson(jsonMap["dateOfBirth"]);
    return i;
  }

  int age = 0;
  Gender gender = Gender.unknown;
  PersonName name = PersonName();
  DateOfBirth dateOfBirth = DateOfBirth();

  @override
  PersonData fromJson(Map<String, dynamic> jsonMap) {
    var i = PersonData();
    i.age = jsonMap["age"];
    i.gender = Gender.values[jsonMap["gender"]];
    i.name = PersonName.fromJson(jsonMap["name"]);
    i.dateOfBirth = DateOfBirth.fromJson(jsonMap["dateOfBirth"]);
    return i;
  }

  @override
  Map<String, dynamic> toJsonMap() {
    var map = {
      'age': age,
      'gender': gender.index,
      'name': name.toJsonMap(),
      'dateOfBirth': dateOfBirth.toJsonMap(),
    };
    return map;
  }

  @override
  String toJson() => jsonEncode(toJsonMap());
}

class PersonName implements JsonSerializer<PersonName> {
  PersonName();

  factory PersonName.fromString(String fullname) {
    var x = PersonName();
    if (fullname.isEmpty) return x;
    var names = fullname.split(" ");
    x.firstName = names.removeAt(0);
    for (var name in names) {
      x.surnames.add(name);
    }
    return x;
  }

  factory PersonName.fromJson(Map<String, dynamic> jsonMap) {
    var i = PersonName();
    i.firstName = jsonMap["firstName"];
    for (var surname in jsonMap["surnames"]) {
      i.surnames.add(surname);
    }
    return i;
  }

  String firstName = "";
  List<String> surnames = [];

  String getFullName() {
    var name = firstName;
    for (var surname in surnames) {
      name = "$name $surname";
    }
    return name;
  }

  @override
  PersonName fromJson(Map<String, dynamic> jsonMap) {
    var i = PersonName();
    i.firstName = jsonMap["firstName"];
    for (var surname in jsonMap["surnames"]) {
      i.surnames.add(surname);
    }
    return i;
  }

  @override
  Map<String, dynamic> toJsonMap() {
    var map = {
      'firstName': firstName,
      'surnames': surnames,
    };
    return map;
  }

  @override
  String toJson() => jsonEncode(toJsonMap());
}

class LegalPersonData extends PersonData {
  LegalPersonData();

  factory LegalPersonData.fromJson(Map<String, dynamic> jsonMap) {
    var i = LegalPersonData();
    i.age = jsonMap["age"];
    i.gender = Gender.values[jsonMap["gender"]];
    i.name = PersonName.fromJson(jsonMap["name"]);
    i.dateOfBirth = DateOfBirth.fromJson(jsonMap["dateOfBirth"]);
    //
    i.address = Address.fromJson(jsonMap["address"]);
    i.phoneNumber = PhoneNumber.fromJson(jsonMap['phoneNumber']);
    i.document = LegalDocument.fromJson(jsonMap["document"]);
    return i;
  }

  Address address = Address();
  PhoneNumber phoneNumber = PhoneNumber();
  LegalDocument document = LegalDocument();

  @override
  LegalPersonData fromJson(Map<String, dynamic> jsonMap) {
    var i = LegalPersonData();
    i.age = jsonMap["age"];
    i.gender = Gender.values[jsonMap["gender"]];
    i.name = PersonName.fromJson(jsonMap["name"]);
    i.dateOfBirth = DateOfBirth.fromJson(jsonMap["dateOfBirth"]);
    //
    i.address = Address.fromJson(jsonMap["address"]);
    i.phoneNumber = PhoneNumber.fromJson(jsonMap["address"]);
    i.document = jsonMap["document"];
    return i;
  }

  @override
  Map<String, dynamic> toJsonMap() {
    var map = {
      'age': age,
      'gender': gender.index,
      'name': name.toJsonMap(),
      'dateOfBirth': dateOfBirth.toJsonMap(),
      'address': address.toJsonMap(),
      'phoneNumber': phoneNumber.toJsonMap(),
      'document': document.toJsonMap(),
    };
    return map;
  }

  @override
  String toJson() => jsonEncode(toJsonMap());
}
