import 'dart:convert';
import 'package:modular_system_core/mscore.dart';

class QueryList<T> implements JsonSerializer<QueryList<T>> {
  QueryList({
    this.objectCreator,
  });

  factory QueryList.fromJsonMap(
      Map<String, dynamic> jsonMap, JsonObjectCreator<T> objectCreator) {
    var list = QueryList<T>();
    list.pagination = PaginationOut.fromJson(jsonMap["pagination"]);
    var data = jsonMap["data"];
    for (var item in data) {
      list.data.add(objectCreator(item));
    }
    return list;
  }

  List<T> data = <T>[];
  PaginationOut pagination = PaginationOut.empty();
  JsonObjectCreator<T>? objectCreator;

  @override
  QueryList<T> fromJson(Map<String, dynamic> jsonMap) {
    if (objectCreator == null) {
      throw AppException(
          "A query list without object creator attemted to deserialize from json map; queyList.fromJson()");
    }
    var list = QueryList<T>();
    list.pagination = PaginationOut.fromJson(jsonMap["pagination"]);
    var data = jsonMap["data"];
    for (var item in data) {
      list.data.add(objectCreator!(item));
    }
    return list;
  }

  @override
  Map<String, dynamic> toJsonMap() {
    return {
      'data': data,
      'pagination': pagination.toJsonMap(),
    };
  }

  @override
  String toJson() => jsonEncode(toJsonMap());
}
