import 'dart:convert';

import 'package:modular_system_core/mscore.dart';

enum WebDataType {
  string,
  boolean,
  int,
  long,
  decimal,
  double,
  float,
  dynamic,
  jsonEncoded,
}

class WebSearchFilter<T> implements JsonSerializer<WebSearchFilter<T>> {
  WebSearchFilter({
    this.name = "",
    this.isActive = false,
    this.type = WebDataType.string,
    this.value,
  });

  factory WebSearchFilter.fromJson(Map<String, dynamic> jsonMap) {
    return WebSearchFilter(
      name: jsonMap["name"],
      isActive: jsonMap["isActive"],
      type: WebDataType.values[jsonMap["type"]],
      value: jsonMap["value"],
    );
  }

  String name;
  bool isActive;
  WebDataType type;
  dynamic value;

  @override
  WebSearchFilter<T> fromJson(Map<String, dynamic> jsonMap) {
    return WebSearchFilter(
      name: jsonMap["name"],
      isActive: jsonMap["isActive"],
      type: WebDataType.values[jsonMap["type"]],
      value: jsonMap["value"],
    );
  }

  @override
  Map<String, dynamic> toJsonMap() {
    return {
      "name": name,
      "isActive": isActive,
      "value": value ?? "",
      "type": type.index,
    };
  }

  @override
  String toJson() => jsonEncode(toJsonMap());
}

class WebSearchRequest implements JsonSerializer<WebSearchRequest> {
  WebSearchRequest({
    required this.pagination,
  });

  factory WebSearchRequest.fromJson(Map<String, dynamic> jsonMap) {
    var i = WebSearchRequest(pagination: PaginationIn.empty());
    i.pagination = PaginationIn.fromJson(jsonMap["pagination"]);
    var data = jsonMap["data"];
    for (var item in data) {
      i.filters.add(WebSearchFilter.fromJson(item));
    }
    return i;
  }

  PaginationIn pagination = PaginationIn.empty();
  List<WebSearchFilter> filters = [];

  @override
  WebSearchRequest fromJson(Map<String, dynamic> jsonMap) {
    var i = WebSearchRequest(pagination: PaginationIn.empty());
    i.pagination = PaginationIn.fromJson(jsonMap["pagination"]);
    var data = jsonMap["data"];
    for (var item in data) {
      i.filters.add(WebSearchFilter.fromJson(item));
    }
    return i;
  }

  @override
  Map<String, dynamic> toJsonMap() {
    var list = <Map<String, dynamic>>[];
    for (var item in filters) {
      list.add(item.toJsonMap());
    }
    return {
      "pagination": pagination.toJsonMap(),
      "filters": list,
    };
  }

  @override
  String toJson() => jsonEncode(toJsonMap());
}

class WebUpdateField implements JsonSerializer<WebUpdateField> {
  WebUpdateField();

  factory WebUpdateField.fromJson(Map<String, dynamic> jsonMap) {
    var i = WebUpdateField();
    i.name = jsonMap['name'];
    i.isActive = jsonMap['isActive'];
    i.value = jsonMap['value'];
    i.type = WebDataType.values[jsonMap['type']];
    return i;
  }

  String name = "";
  bool isActive = false;
  dynamic value;
  WebDataType type = WebDataType.string;

  @override
  WebUpdateField fromJson(Map<String, dynamic> jsonMap) {
    return WebUpdateField.fromJson(jsonMap);
  }

  @override
  Map<String, dynamic> toJsonMap() {
    return {
      'name': name,
      'isActive': isActive,
      'value': value,
      'type': type.index,
    };
  }

  @override
  String toJson() => jsonEncode(toJsonMap());
}

class WebUpdateRequest implements JsonSerializer<WebUpdateRequest> {
  WebUpdateRequest({
    required this.id,
  });

  String id = "";
  List<WebUpdateField> updates = [];

  @override
  WebUpdateRequest fromJson(Map<String, dynamic> jsonMap) {
    // TODO: implement fromJson
    throw UnimplementedError();
  }

  @override
  Map<String, dynamic> toJsonMap() {
    var list = <Map<String, dynamic>>[];
    for (var item in updates) {
      list.add(item.toJsonMap());
    }
    return {
      'id': id,
      'updates': list,
    };
  }

  @override
  String toJson() => jsonEncode(toJsonMap());
}
