import 'package:flutter/material.dart';
import 'package:modular_system_core/mscore.dart';

typedef TableWidgetBuilder<T> = Widget Function(BuildContext context, EntityTableViewModel<T> vm);
typedef TableItemBuilder<T> = Widget Function(BuildContext context, T data, EntityTableViewModel<T> vm);
typedef TableEditorBuilder<T> = Widget Function(BuildContext context, T data, EntityTableViewModel<T> vm);

class EntityTableViewModelOld<T> {
  bool extraOptionsOn = false;
  bool hasExtraOptions = false;
  bool selectorOn = false;
  TextEditingController searchbarController = TextEditingController();
  AppEvent updateEvent = AppEvent();
  AppEvent showForm = AppEvent();
  AppEvent showTable = AppEvent();
  AppEvent showExtraOptionsEvent = AppEvent();
  AppEvent selectorStateChangedEvent = AppEvent();
  AppEvent tileSelectedEvent = AppEvent();
  WebSearchRequest search = WebSearchRequest(pagination: PaginationIn.fromLimit());
  PaginationOut paginationOut = PaginationOut.fromLimit();
  List<T> data = [];
  List<int> selectedTiles = [];
}

class EntityTableViewModel<T> {
  EntityTableViewModel({
    required this.properties,
    required this.config,
  });

  TableVmProperties<T> properties;
  TableVmConfig config;
  TableVmState<T> state = TableVmState();
  TableVmEvents<T> events = TableVmEvents();
}

class TableVmConfig {
  TableVmConfig({
    required this.hasAppbarActions,
    required this.searchbarAutoUpdate,
  });

  bool hasAppbarActions = false;
  bool searchbarAutoUpdate = false;
}

class TableVmState<T> {
  bool extraOptionsOn = false;
  bool selectorOn = false;
  TextEditingController searchbarController = TextEditingController();
  late Future updateFuture;
  WebSearchRequest search = WebSearchRequest(pagination: PaginationIn.fromLimit());
  PaginationOut paginationOut = PaginationOut.fromLimit();
  List<T> data = [];
  List<int> selectedTiles = [];

  TableVmState setUpdateFuture(Future function) {
    updateFuture = function;
    return this;
  }
}

class TableVmProperties<T> {
  TableVmProperties({
    required this.update,
    required this.itemBuilder,
    required this.formBuilder,
    required this.transitionDuration,
    required this.filtersSheetBuilder,
    this.editorBuilder,
    this.viewerBuilder,
    this.selectorActionsBuilder,
    this.appbarActions,
  });

  // dependency injection
  late Future<SingleEntryMap<PaginationOut, List<T>>> Function(WebSearchRequest search) update;

  late TableItemBuilder<T> itemBuilder;
  late TableWidgetBuilder<T> formBuilder;

  late Duration transitionDuration;
  // optional
  late Widget Function(EntityTableViewModel<T> tableViewModel, T instance)? viewerBuilder;
  late TableEditorBuilder<T>? editorBuilder;
  late AppBottomSheet Function(EntityTableViewModel<T> viewModel) filtersSheetBuilder;

  late List<SingleEntryMap<Widget, Function>> Function(EntityTableViewModel<T> viewModel)? selectorActionsBuilder;

  late List<SingleEntryMap<IconData, Function(EntityTableViewModel vm)>>? appbarActions;
}

class TableVmEvents<T> {
  Event update = Event();
  Event showTable = Event();
  Event showForm = Event();
  Event showExtraOptionsEvent = Event();
  Event searchFiltersChanged = Event();
  GetEvent<String> getSearchText = GetEvent();
  GetEvent<List<WebSearchFilter>> getSearchFilters = GetEvent();
  SetEvent<T> openViewer = SetEvent();
  SetEvent setSelectedTiles = SetEvent();
  SetEvent<List<T>> setData = SetEvent();
  SetEvent<PaginationOut> setPagination = SetEvent();
  SetEvent<bool> setSelectorMode = SetEvent();
  SetEvent<String> setSearchText = SetEvent();
  SetEvent<List<WebSearchFilter>> setSearchFilters = SetEvent();
  // deprecated; to be refactored
  AppEvent<T, dynamic> showEditor = AppEvent();
}
